# -*- python -*-
# This file is part of Fusion-icon.

# Fusion-icon is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Fusion-icon is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author(s): crdlb, nesl247
#
# Copyright 2007 Christopher Williams <christopherw@verizon.net> 

import sys, os
from FusionIcon.util import env
from FusionIcon import start

interfaces={
	'qt': 'Qt',
	'gtk': 'GTK+',
}

def import_interface(interface):
	try:
		if interface in interfaces:
			print(' * Using the ' + interfaces[interface] + ' Interface')
			__import__('FusionIcon.interface_' + interface)

		else:
			print(' *** Error: "' + interface + '" interface is invalid, this should not happen')
			raise SystemExit

	except ImportError as e:
		if [i for i in interfaces if 'interface_' + i in str(e)]:
			print(' * Interface not installed')
		else:
			print(' * ' + str(e))

		#doesn't work so remove it from the dict
		del interfaces[interface]
		if interfaces:
			print(' ... Trying another interface')
			choose_interface()
		else:
			print(' *** Error: All interfaces failed, aborting!')
			raise SystemExit

def choose_interface(try_first=None):

	chosen_interface = None

	# handle explicit choice first
	if try_first:
		if try_first in interfaces:
			chosen_interface = try_first
		else:
			raise SystemExit(' *** Error: No such interface: ' + try_first)
	else:

		# use qt for kde and lxqt; gtk for everything else:
		if os.getenv('FUSION_ICON_INTERFACE') in interfaces:
			chosen_interface = os.getenv('FUSION_ICON_INTERFACE')
		elif 'qt' in interfaces and env.desktop in ('kde', 'lxqt'):
			chosen_interface = 'qt'
		elif 'gtk' in interfaces:
			chosen_interface = 'gtk'
		elif 'qt' in interfaces:
			chosen_interface = 'qt'
		# interfaces is empty
		else:
			raise SystemExit(' *** no available interfaces, this should not happen')

	import_interface(chosen_interface)



# -*- python -*-
# This file is part of Fusion-icon.

# Fusion-icon is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Fusion-icon is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Publaic License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Original copyright 2007 Christopher Williams <christopherw@verizon.net> 
# Author(s): crdlb, kozec, raveit65

import os

mesa_libgl_locations = (
	# ubuntu
	'/usr/lib/fglrx/libGL.so.1.2.xlibmesa',
	'/usr/lib/nvidia/libGL.so.1.2.xlibmesa',
	# gentoo
	'/usr/lib/opengl/xorg-x11/lib/libGL.so.1.2',
	# archlinux
	'/opt/mesa-xgl/lib/libGL.so.1.2',
	'/lib/mesa/libGL.so.1.2',
	# debian
	'/usr/lib/fglrx/diversions/libGL.so.1.2',
	'/usr/share/nvidia-glx/diversions/libGL.so.1.2',
)

compiz_args = ['ccp', '--replace', '--sm-disable', '--ignore-desktop-hints']

config_home = os.environ.get('XDG_CONFIG_HOME',
			os.path.join(os.environ['HOME'], '.config'))

config_folder = os.path.join(config_home, 'compiz')

config_file = os.path.join(config_folder, 'fusion-icon')

#app
#	base command, full command line
#	label

apps = {
	'ccsm':
		('ccsm', ['ccsm'],
		'Settings Manager'),
	'emerald theme manager':
		('emerald-theme-manager', ['emerald-theme-manager'],
		'Emerald Theme Manager'),
}

#wm
#	base command, full command line
#	label, desktop, special flags, command to run before replacing

wms = {
	'marco':
		('marco', ['marco', '--replace'],
		 'Marco', 'mate', None, None,),

	'metacity':
		('metacity', ['metacity', '--replace'],
		 'Metacity', 'gnome', None, None,),

	'mutter':
		('mutter', ['mutter', '--replace'],
		 'Mutter', 'gnome', None, None,),

	'kwin':
		('kde', ['kwin', '--replace'],
		 'KWin', 'kde', None, None),

	'xfwm4':
		('xfwm4', ['xfwm4'],
		 'Xfwm4', 'xfce', ['noreplace'], ['killall', 'xfwm4']),

	'openbox':
		('openbox', ['openbox', '--replace'],
		 'Openbox', None, None, None),

	'blackbox':
		('blackbox', ['blackbox', '--replace'],
		 'Blackbox', None, None, None),

	'fvwm':
		('fvwm', ['fvwm', '--replace'],
		 'FVWM', None, None, None),

	'icewm':
		('icewm', ['icewm', '--replace'],
		 'IceWM', None, None, None),

}

#decorator
#	base command, full command line,
#	label, desktop environment

decorators = {
	'emerald':
		('emerald', 'emerald --replace',
		 'Emerald', None),

	'gwd':
		('gtk-window-decorator', 'gtk-window-decorator --replace',
		 'GTK+ Window Decorator', 'mate'),
}

#option:
#	(unused), switch,
#	label

options = {
	'indirect rendering':
		(None, '--indirect-rendering', 'Indirect Rendering'),

	'loose binding':
		(None, '--loose-binding', 'Loose Binding'),
}



# -*- python -*-
# This file is part of Fusion-icon.

# Fusion-icon is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Fusion-icon is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Original copyright 2007 Christopher Williams <christopherw@verizon.net>
# Author(s): crdlb, nesl247, raveit65

from FusionIcon.parser import options as parser_options
from FusionIcon.util import env, config, apps, options, wms, decorators

def init():
	'Final start function, should be called once when fusion-icon starts'

	if not parser_options.no_start:
		# Do not restart the wm if it's already running
		if wms.active == wms.active == wms.old != 'compiz':
			#always restart compiz since we can't know compiz was started correctly
			print(' * ' + wms[wms.active].label + ' is already running')
		else:
			print(' * Starting ' + wms[wms.active].label)
			wms.start()

config.check()

# Make some changes

if not parser_options.force_compiz:
	if wms.active not in wms:
		print(' * "' + wms.active + '" not installed')
		if wms.fallback:
			print(' ... setting to fallback...')
		else:
			print(' ... No fallback window manager chosen')
		wms.active = wms.fallback

elif 'compiz' in wms:
	wms.active = 'compiz'

else:
	raise SystemExit(' *** Error: "--force-compiz" used and compiz not installed!')

# Set True if using Xorg AIGLX since the '--indirect-rendering' option has no effect in that situation. 
env.set()
if env.tfp == 'indirect' and 'indirect rendering' in options:
	options['indirect rendering'].sensitive = False
	if not options['indirect rendering'].enabled:
		options['indirect rendering'].enabled = True



# -*- python -*-
# This file is part of Fusion-icon.

# Fusion-icon is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Fusion-icon is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Original copyright 2007 Christopher Williams <christopherw@verizon.net>
# Author(s): crdlb, nesl247, raveit65

import os
from FusionIcon.execute import run

tfp = 'GLX_EXT_texture_from_pixmap'

class Environment:

	'''Detects properties of the enviroment, and provides a set() method that uses this information to export environment variables needed to start compiz.'''


	def __init__(self):

		'''desktop: current desktop enviroment used to choose interface, fallback wm, and default decorator

glxinfo: output of glxinfo command

indirect_glxinfo: output of glxinfo with LIBGL_ALWAYS_INDIRECT

xvinfo: output of xvinfo

glx_vendor: 'client glx vendor:' usually one of SGI (for mesa-based drivers), NVIDIA Corporation, or ATI.

tfp: 'direct' if texture_from_pixmap is present with direct rendering (implying presence with indirect as well), 'indirect' if only present with indirect context, False if not present at all

Xgl: True in Xgl'''

		# Check for various desktop environments
		if os.getenv('XDG_CURRENT_DESKTOP') == 'MATE' or os.getenv('MATE_DESKTOP_SESSION_ID') is not None:
			self.desktop = 'mate'
		elif os.getenv('XDG_CURRENT_DESKTOP') == 'XFCE' or os.getenv('DESKTOP_SESSION') in ('xfce', 'xfce4', 'Xfce Session'):
			self.desktop = 'xfce'
		elif os.getenv('XDG_CURRENT_DESKTOP', '').endswith('GNOME') or os.getenv('GNOME_DESKTOP_SESSION_ID') is not None:
			self.desktop = 'gnome'
		elif os.getenv('XDG_CURRENT_DESKTOP') == 'KDE' or os.getenv('KDE_FULL_SESSION') is not None:
			self.desktop = 'kde'
		elif os.getenv('XDG_CURRENT_DESKTOP') == 'LXQt':
			self.desktop = 'lxqt'
		elif os.getenv("XDG_CURRENT_DESKTOP") is not None:
			self.desktop = os.getenv('XDG_CURRENT_DESKTOP', 'unknown').lower()
		else:
			self.desktop = os.getenv('DESKTOP_SESSION', 'unknown').lower()

		print(' * Detected Session: ' + self.desktop)


		## Save the output of glxinfo and xvinfo for later use:

		# don't try to run glxinfo unless it's installed
		if run(['which', 'glxinfo'], 'call', quiet=True) == 0:
			self.glxinfo = run('glxinfo', 'output')
		else:
			raise SystemExit(' * Error: glxinfo not installed!')

		# make a temp environment
		indirect_environ = os.environ.copy()
		indirect_environ['LIBGL_ALWAYS_INDIRECT'] = '1'
		self.indirect_glxinfo = run('glxinfo', 'output', env=indirect_environ)

		if run(['which', 'xvinfo'], 'call', quiet=True) == 0:
			self.xvinfo = run('xvinfo', 'output')
		else:
			raise SystemExit(' * Error: xvinfo not installed!')

		self.glx_vendor = None
		line = [l for l in self.glxinfo.splitlines() if 'client glx vendor string:' in l]
		if line:
			self.glx_vendor = ' '.join(line[0].split()[4:])

		## Texture From Pixmap / Indirect
		self.tfp = False
		if self.glxinfo.count(tfp) < 3:
			if self.indirect_glxinfo.count(tfp) == 3:
				self.tfp = 'indirect'
		else:
			self.tfp = 'direct'

		## Xgl
		if 'Xgl' in self.xvinfo:
			self.Xgl = True

		else:
			self.Xgl = False

	def set(self):

		'''Trigger all environment checks'''

		# Check for Intel and export INTEL_BATCH
		if 'Intel' in self.glxinfo:
			print(' * Intel detected, exporting: INTEL_BATCH=1')
			os.environ['INTEL_BATCH'] = '1'

		# Check TFP and export LIBGL_ALWAYS_INDIRECT if needed
		if self.tfp != 'direct':
			print(' * No ' + tfp + ' with direct rendering context')
			if self.tfp == 'indirect':
				print(' ... present with indirect rendering, exporting: LIBGL_ALWAYS_INDIRECT=1')
				os.environ['LIBGL_ALWAYS_INDIRECT'] = '1'
			else:
				# If using Xgl with a proprietary driver, exports LD_PRELOAD=<mesa libGL>
				if self.Xgl and self.glx_vendor != 'SGI':
					print(' * Non-mesa driver on Xgl detected')
					from data import mesa_libgl_locations
					location = [l for l in mesa_libgl_locations if os.path.exists(l)]
					if location:
						print(' ... exporting: LD_PRELOAD=' + location[0])
						os.environ['LD_PRELOAD'] = location[0]
						if run(['glxinfo'], 'output').count(tfp) >= 3:
							self.tfp = 'direct'

					else:
						# kindly let the user know... but don't abort (maybe it will work :> )
						print(' ... no mesa libGL found for preloading, this may not work!')
				else:
					print(' ... nor with indirect rendering, this isn\'t going to work!')

		# Check for nvidia on Xorg
		if not self.Xgl and self.glx_vendor == 'NVIDIA Corporation':
			print(' * NVIDIA on Xorg detected, exporting: __GL_YIELD=NOTHING')
			os.environ['__GL_YIELD'] = 'NOTHING'

env = Environment()



# -*- python -*-
# This file is part of Fusion-icon.

# Fusion-icon is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Fusion-icon is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Original copyright 2007 Christopher Williams <christopherw@verizon.net>
# Author(s): crdlb, nesl247, raveit65

import subprocess, signal, os

# avoid zombies
signal.signal(signal.SIGCHLD, signal.SIG_IGN)

def run(command, mode='spawn', quiet=False, env=None):
	'Simple wrapper for the subprocess module. Supported modes: spawn, call, and output'

	try:
		if mode == 'spawn':
			if not quiet:
				popen_object = subprocess.Popen(command)
			else:
				popen_object = subprocess.Popen(command, stdout=open(os.devnull, 'w'))

			return popen_object

		elif mode == 'call':
			# restore normal child handling
			signal.signal(signal.SIGCHLD, signal.SIG_DFL)
			if not quiet:
				exitcode = subprocess.call(command, stderr=subprocess.PIPE)
			else:
				exitcode = subprocess.call(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

			# turn zombie protection back on
			signal.signal(signal.SIGCHLD, signal.SIG_IGN)

			return exitcode

		elif mode == 'output':
			signal.signal(signal.SIGCHLD, signal.SIG_DFL)
			if not env:
				output = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=open(os.devnull, "w"), universal_newlines=True).communicate()[0]
			else:
				output = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=open(os.devnull, "w"), universal_newlines=True, env=env).communicate()[0]

			signal.signal(signal.SIGCHLD, signal.SIG_IGN)

			return output

	except OSError:
		print(' * execution of "' + ' '.join(command) + '" failed')
		return None



# -*- python -*-
# This file is part of Fusion-icon.

# Fusion-icon is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Fusion-icon is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Original copyright 2007 Christopher Williams <christopherw@verizon.net>
# Author(s): crdlb, kozec, raveit65

from optparse import OptionParser, OptionGroup

parser = OptionParser(usage='usage: %prog [options|action]', version='%prog-0.0.0')

parser.add_option('--reset', action='store_true', dest='reset',
	help='remove configuration file and exit')

parser.add_option('-s', '--sleep', type='int', dest='seconds',
	help='Sleep before launching')

parser.add_option('-v', '--verbose', action='store_true', dest='verbose',
	help='Print extra output')

interface_group = OptionGroup(parser, 'Interface Options')

interface_group.add_option('-i', '--interface', dest='interface',
	help='Try a certain interface first')

interface_group.add_option('-u', '--no-interface', action='store_true', dest='no_interface',
	help='Do not use any interface')

parser.add_option_group(interface_group)

startup_group = OptionGroup(parser, 'Startup Options')

startup_group.add_option('-f', '--force-compiz', action='store_true', dest='force_compiz',
	help='Start compiz regardless of environment or configuration')

startup_group.add_option('-n', '--no-start', action='store_true', dest='no_start',
	help='Run, but do not start a window manager')

parser.add_option_group(startup_group)

options, args = parser.parse_args()

# fusion-icon accepts no arguments
if args:
	parser.error('no such argument: ' + args[0])



# -*- python -*-
# This file is part of Fusion-icon.

# Fusion-icon is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Fusion-icon is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Based on compiz-icon, Copyright 2007 Felix Bellanger <keeguon@gmail.com>
#
# Original copyright 2007 Christopher Williams <christopherw@verizon.net>
# Author(s): crdlb, kozec, raveit65

import os, compizconfig, time
try:
    import configparser
except ImportError:
    # Python 2.x compatibility.
    import ConfigParser as configparser

import FusionIcon.data as _data
from FusionIcon.parser import options as parser_options
from FusionIcon.environment import env
from FusionIcon.execute import run
import subprocess, signal

def is_running(app):
	'Use pgrep to determine if an app is running'

	if run(['pgrep', app], 'call', quiet=True) == 0:
		return True


class Application(object):

	def __init__(self, name, apps, installed):

		self.name = name
		self.apps = apps

		self.base = installed.apps[name][0]
		self.command = installed.apps[name][1]
		self.label = installed.apps[name][2]

	def launch(self):
		print(' * Launching ' + self.label)
		run(self.command)

class Applications(dict):

	def __init__(self, installed):
		for app in installed.apps:
			self[app] = Application(app, self, installed)

class CompizOption(object):

	def __init__(self, name, options, installed, config):

		self.options = options
		self.config = config

		self.name = name
		self.switch = installed.options[name][1]
		self.label = installed.options[name][2]
		self.sensitive = True

	def __get(self):
		return self.config.getboolean('compiz options', self.name)

	def __set(self, value):
		print(' * Setting option ' + self.label + ' to ' + value)
		self.config.set('compiz options', self.name, str(bool(value)).lower())
		self.config.write(open(self.config.config_file, 'w'))

	enabled = property(__get, __set)

class CompizOptions(dict):

	def __init__(self, installed, config):
		for option in installed.options:
			self[option] = CompizOption(option, self, installed, config)

class WindowManager(object):

	def __init__(self, name, wms, installed):

		self.wms = wms
		self.name = name
		self.base = installed.wms[name][0]
		self.command = installed.wms[name][1]
		self.label = installed.wms[name][2]
		self.desktop = installed.wms[name][3]
		if installed.wms[name][4]:
			self.flags = installed.wms[name][4]
		else:
			self.flags = []
		self.killcmd = installed.wms[name][5]

class WindowManagers(dict):

	def __init__(self, installed, config):

		self.config = config

		for wm in installed.wms:
			self[wm] = WindowManager(wm, self, installed)

		self.fallback = None
		wm = [w for w in self if self[w].desktop == env.desktop]
		if wm:
			self.fallback = wm[0]

		elif [w for w in self if w != 'compiz']:
			self.fallback = list([w for w in self if w != 'compiz'])[0]

		self.__set_old()

		self.ordered_list = []
		for wm in ('compiz', self.fallback):
			if wm in self:
				self.ordered_list.append(wm)
		self.ordered_list.extend([wm for wm in self if wm not in self.ordered_list])

	def __get(self):
		return self.config.get('window manager', 'active wm')

	def __set(self, value):

		if value in wms:
			print(' * Setting window manager to ' + wms[value].label)
		elif not value:
			print(' * Setting window manager to empty value')

		self.config.set('window manager', 'active wm', str(value))
		self.config.write(open(self.config.config_file, 'w'))

	def __set_old(self):

		self.old = None
		running_wm = [wm for wm in self if is_running(wm)]
		if running_wm:
			# not perfect, but good enough
			self.old = running_wm[0]

	def start(self):
		'Start the active window manager'

		self.__set_old()

		if self.active == 'compiz' and self.old and self[self.old].killcmd:
			if run(['which', self[self.old].killcmd[0]], 'call', quiet=True) == 0:
				run(self[self.old].killcmd, 'call')
				time.sleep(1)

		if self.active and self.old and 'noreplace' in self[self.active].flags:
			run(['killall', self[self.old].base], 'call')
			time.sleep(1)

		if self.active == 'compiz':
			# use a copy, not the original
			compiz_command = self['compiz'].command[:]
			for option in options:
				if options[option].enabled:
					if options[option].switch is not None:
						compiz_command.append(options[option].switch)

			kill_list = ['killall']
			for decorator in decorators:
				kill_list.append(decorators[decorator].base)
			run(kill_list, 'call')

			time.sleep(0.5)

			# do it
			print(' ... executing: ' + ' '.join(compiz_command))
			run(compiz_command, quiet=False)

		elif self.active:
			run(self[self.active].command)

		else:
			print(' * No active WM set; not going to do anything.')

	def restart(self):
		if wms.active:
			print(' * Reloading ' + wms.active)
			self.start()

		else:
			print(' * Not reloading, no active window manager set')

	active = property(__get, __set)

class CompizDecorator(object):

	def __init__(self, name, decorators, installed):

		self.decorators = decorators
		self.name = name
		self.base = installed.decorators[name][0]
		self.command = installed.decorators[name][1]
		self.label = installed.decorators[name][2]
		self.desktop = installed.decorators[name][3]

	def kill_others(self):
		killall = ['killall']
		for decorator in [x for x in self.decorators if x != self.name]:
			killall.append(self.decorators[decorator].base)
		run(killall, 'call')

class CompizDecorators(dict):

	def __init__(self, installed):

		# Open CompizConfig context
		if parser_options.verbose:
			print(' * Opening CompizConfig context')

		context = compizconfig.Context()

		# Either Compiz 0.9.x or Compiz 0.8.x.
		if 'decor' in context.Plugins:
			self.command = context.Plugins['decor'].Screen['command']
		else:
			self.command = context.Plugins['decoration'].Display['command']

		for decorator in installed.decorators:
			self[decorator] = CompizDecorator(decorator, self, installed)

		self.default = None
		decorator = [d for d in self if self[d].desktop == env.desktop]
		if decorator:
			self.default = decorator[0]

		elif 'emerald' in self:
			self.default = 'emerald'

		elif self:
			self.default = next(iter(self))

	def __set(self, decorator):
		if decorator in self:
			decorators[decorator].kill_others()
			time.sleep(0.5)
			self.command.Plugin.Context.ProcessEvents()
			print(' * Setting decorator to ' + self[decorator].label + \
				' ("' + self[decorator].command + '")')
			self.command.Value = self[decorator].command
			self.command.Plugin.Context.Write()
		elif not decorator:
			print(' * Not setting decorator to none')

	def __get(self):
		_decorator = [d for d in self if self.command.Value == self[d].command]
		if _decorator:
			decorator = _decorator[0]
		else:
			print(' * Decorator "' + self.command.Value + '" is invalid.')
			self.active = self.default
			decorator = self.command.Value
		return decorator

	active = property(__get, __set)

class Installed(object):

	def __init__(self, data):
		print(' * Searching for installed applications...')

		### Compiz Detection
		bins = {}
		for name in ('compiz', 'compiz.real'):
			bin = run(['which', name], 'output')
			if bin:
				bins[name] = bin

		if 'compiz' in bins and 'compiz.real' in bins:
			if bins['compiz'].split(os.sep)[:-1] == bins['compiz.real'].split(os.sep)[:-1]:
				compiz = 'compiz.real'
			else:
				compiz = 'compiz'

		elif 'compiz.real' in bins:
			compiz = 'compiz.real'

		elif 'compiz' in bins:
			compiz = 'compiz'

		else:
			compiz = None

		output = ''

		for name in bins:
			if len(bins) > 1 and name == compiz:
				selected = ' <*>'
			else:
				selected = ''
			output += ' -- ' + bins[name] + selected

		### Everything Else
		self.wms = data.wms.copy()
		for wm in data.wms:
			which = run(['which', data.wms[wm][0]], 'output')
			if which:
				output += ' -- ' + which
			else:
				del self.wms[wm]

		if compiz:
			data.compiz_args.insert(0, compiz)
			self.wms['compiz'] = (compiz, data.compiz_args, 'Compiz', None, None, None)

		self.decorators = data.decorators.copy()
		for decorator in data.decorators:
			which = run(['which', data.decorators[decorator][0]], 'output')
			if which:
				output += ' -- ' + which
			else:
				del self.decorators[decorator]

		self.apps = data.apps.copy()
		for app in data.apps:
			which = run(['which', data.apps[app][0]], 'output')
			if which:
				output += ' -- ' + which
			else:
				del self.apps[app]

		if parser_options.verbose:
			print(output.rstrip())

		compiz_optionlist = []

		self.options = data.options.copy()

		if compiz:
			compiz_help = run([compiz, '--help'], 'output')
			for item in compiz_help.split():
				item = item[1:].strip(" \r\n\t]")
				if item.startswith('--'):
					compiz_optionlist.append(item)

		for option in data.options:
			if data.options[option][1] not in compiz_optionlist:
				del self.options[option]

class Configuration(configparser.ConfigParser):

	def __init__(self, data):

		configparser.ConfigParser.__init__(self)
		self.config_folder = data.config_folder
		self.config_file = data.config_file

	def check(self):

		# Configuration file setup
		if not os.path.exists(self.config_folder):
			if parser_options.verbose: 
				print(' * Creating configuration folder...')
			os.makedirs(self.config_folder)

		if not os.path.exists(self.config_file):
			if parser_options.verbose:
				print(' * Creating configuration file...')
			self.create_config_file()

		try:
			# Read the file
			self.read(self.config_file)
			# Validate the file by trying to read all values
			for option in options:
				value = options[option].enabled
			value = wms.active

		except:
			# back it up and make a new one
			print(' * Configuration file (' + self.config_file + ') invalid')
			self.reset_config_file()
			print(' * Generating new configuration file')
			self.create_config_file()

	def create_config_file(self):
		'Set default values for configuration file'

		def prune(section, optlist):
			for option in [o for o in self.options(section) if o not in optlist]:
				self.remove_option(section, option)

		for section in ('compiz options', 'window manager'):
			if not self.has_section(section):
				self.add_section(section)

		for option in options:
			self.set('compiz options', option, 'false')
		self.set('window manager', 'active wm', 'compiz')

		prune('compiz options', options)
		prune('window manager', ['active wm'])
		self.write(open(self.config_file, 'w'))

	def reset_config_file(self):
		'Backup configuration file'

		if os.path.exists(self.config_file):
			config_backup = '%s.backup.%s' \
				%(self.config_file, time.strftime('%Y%m%d%H%M%S'))
			os.rename(self.config_file, config_backup)
			print(' ... backed up to: ' + config_backup)
		else:
			print(' ... no configuration file found')

# Instantiate...
_installed = Installed(_data)
config = Configuration(_data)
apps = Applications(_installed)
options = CompizOptions(_installed, config)
wms = WindowManagers(_installed, config)
decorators = CompizDecorators(_installed)



#!/usr/bin/env python

import sys, os
from stat import *
from distutils.core import setup
from distutils.command.install import install as _install

INSTALLED_FILES = '.installed_files'

#stolen from ccsm
class install (_install):

	def run (self):

		_install.run(self)
		outputs = self.get_outputs()
		data = '\n'.join(outputs)
		try:
			f = open(INSTALLED_FILES, 'w')
		except:
			self.warn ('Could not write installed files list ' + INSTALLED_FILES)
			return

		f.write(data)
		f.close()

class uninstall(_install):

	def run(self):
		try:
			files = file(INSTALLED_FILES, 'r').readlines()
		except:
			self.warn('Could not read installed files list ' + INSTALLED_FILES)
			return

		for f in files:
			print('Uninstalling ' + f.strip())
			try:
				os.unlink(f.strip())
			except:
				self.warn('Could not remove file ' + f)
		os.remove(INSTALLED_FILES)

#Stolen from ccsm's setup.py
if sys.argv[1] == 'build':

	qtver = '5.0'
	gtkver = '3.0'

	if len (sys.argv) > 2:
		i = 0
		for o in sys.argv:
			if o.startswith('--with-qt'):
				if o == '--with-qt':
					if len(sys.argv) >= i:
						qtver = sys.argv[i + 1]
					sys.argv.remove(qtver)
				elif o.startswith('--with-qt=') and len(o[10:]):
					qtver = o[10:]
				sys.argv.remove(o)
			i += 1
		i = 0
		for o in sys.argv:
			if o.startswith('--with-gtk'):
				if o == '--with-gtk':
					if len(sys.argv) >= i:
						gtkver = sys.argv[i + 1]
					sys.argv.remove(gtkver)
				elif o.startswith('--with-gtk=') and len(o[11:]):
					gtkver = o[11:]
				sys.argv.remove(o)
			i += 1

	f = open(os.path.join ('FusionIcon/interface_qt/main.py.in'), 'rt')
	data = f.read()
	f.close()
	data = data.replace('@qtver@', qtver.split('.')[0])
	f = open(os.path.join('FusionIcon/interface_qt/main.py'), 'wt')
	f.write(data)
	f.close()

	f = open(os.path.join ('FusionIcon/interface_gtk/main.py.in'), 'rt')
	data = f.read()
	f.close()
	data = data.replace('@gtkver@', gtkver)
	if gtkver == '2.0':
		data = data.replace('@aiver@', '')
	else:
		data = data.replace('@aiver@', '3')
	f = open(os.path.join('FusionIcon/interface_gtk/main.py'), 'wt')
	f.write(data)
	f.close()


version = open('VERSION', 'r').read().strip()

packages = ['FusionIcon']

available_interfaces = {
	'qt': 'FusionIcon.interface_qt',
	'gtk': 'FusionIcon.interface_gtk',
}

#if 'interfaces' in os.environ:
# 	for interface in os.environ['interfaces'].split():
#		if interface in available_interfaces:
#			packages.append(available_interfaces[interface])
#else:

packages.extend(available_interfaces.values())


data_files = [
	('share/icons/hicolor/22x22/apps',['images/22x22/fusion-icon.png']),
	('share/icons/hicolor/24x24/apps',['images/24x24/fusion-icon.png']),
	('share/icons/hicolor/32x32/apps',['images/32x32/fusion-icon.png']),
	('share/icons/hicolor/48x48/apps',['images/48x48/fusion-icon.png']),
	('share/icons/hicolor/scalable/apps',['images/scalable/fusion-icon.svg']),
	('share/applications',['fusion-icon.desktop']),
	('share/metainfo',['fusion-icon.appdata.xml']),
]


setup(
	name='fusion-icon',
	version=version,
	description='User-friendly tray icon for launching and managing Compiz',
	author='Wolfgang Ulbrich',
	author_email='chat-to-me@raveit.de',
	url='https://github.com/compiz-reloaded/fusion-icon',
	packages=packages,
	scripts=['fusion-icon'],
	data_files=data_files,
	cmdclass={
		'uninstall': uninstall,
		'install': install},
)

#Stolen from ccsm's setup.py
if sys.argv[1] == 'install':

	prefix = None

	if len (sys.argv) > 2:
		i = 0
		for o in sys.argv:
			if o.startswith ("--prefix"):
				if o == "--prefix":
					if len (sys.argv) >= i:
						prefix = sys.argv[i + 1]
					sys.argv.remove (prefix)
				elif o.startswith ("--prefix=") and len (o[9:]):
					prefix = o[9:]
				sys.argv.remove (o)
				break
			i += 1

	if not prefix:
		prefix = '/usr/local'

	gtk_update_icon_cache = '''gtk-update-icon-cache -f -t \
%s/share/icons/hicolor''' % prefix
	root_specified = [s for s in sys.argv if s.startswith('--root')]
	if not root_specified or root_specified[0] == '--root=/':
		print('Updating Gtk icon cache.')
		os.system(gtk_update_icon_cache)
	else:
		print('''*** Icon cache not updated. After install, run this:
***     %s''' % gtk_update_icon_cache)
